@@ -21,7 +21,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <stdint.h>
+#include <_stdint.h>
 #include <stdlib.h>
 #include <time.h>
 #include <fcntl.h>
@@ -34,6 +34,9 @@
 
 #include <gphoto2/gphoto2-result.h>
 #include "ax203.h"
+#ifdef HAVE_LIBJPEG
+#include "jpeg_memsrcdest.h"
+#endif
 
 static const struct eeprom_info {
 	const char *name;
@@ -177,6 +180,44 @@ ax203_get_lcd_size(Camera *camera)
 }
 #endif
 
+static int
+ax3003_get_frame_id(Camera *camera)
+{
+	int ret;
+	char cmd_buffer[16];
+	uint8_t id;
+
+	memset (cmd_buffer, 0, sizeof (cmd_buffer));
+
+	cmd_buffer[0] = AX3003_FRAME_CMD;
+	cmd_buffer[1] = AX3003_GET_FRAME_ID;
+
+	ret = ax203_send_cmd (camera, 0, cmd_buffer, sizeof(cmd_buffer),
+			      (char *)&id, 1);
+	if (ret < 0) return ret;
+
+	return id;
+}
+
+static int
+ax3003_get_abfs_start(Camera *camera)
+{
+	int ret;
+	char cmd_buffer[16];
+	uint8_t buf[2];
+
+	memset (cmd_buffer, 0, sizeof (cmd_buffer));
+
+	cmd_buffer[0] = AX3003_FRAME_CMD;
+	cmd_buffer[1] = AX3003_GET_ABFS_START;
+
+	ret = ax203_send_cmd (camera, 0, cmd_buffer, sizeof(cmd_buffer),
+			      (char *)buf, 2);
+	if (ret < 0) return ret;
+
+	return be16atoh(buf) * 0x100;
+}
+
 int
 ax203_set_time_and_date(Camera *camera, struct tm *t)
 {
@@ -188,13 +229,19 @@ ax203_set_time_and_date(Camera *camera, struct tm *t)
 	
 	cmd_buffer[5] = t->tm_year % 100;
 
-	switch (camera->pl->firmware_version) {
+	switch (camera->pl->frame_version) {
+	case AX3003_FIRMWARE_3_5_x:
+		/* Note this is what the windows software does, with the
+		   one AX3003 frame I have this does not do anything */
+		cmd_buffer[0] = AX3003_FRAME_CMD;
+		cmd_buffer[1] = AX3003_SET_TIME;
+		/* fall through */
 	case AX203_FIRMWARE_3_3_x:
 	case AX203_FIRMWARE_3_4_x:
 		cmd_buffer[6] = t->tm_mon + 1;
 		cmd_buffer[7] = t->tm_wday;
 		break;
-	case AX203_FIRMWARE_3_5_x:
+	case AX206_FIRMWARE_3_5_x:
 		cmd_buffer[6] = 19 + t->tm_year / 100;
 		cmd_buffer[7] = t->tm_mon + 1;
 		break;
@@ -260,6 +307,17 @@ ax203_eeprom_write_enable(Camera *camera)
 	return ax203_send_eeprom_cmd (camera, 1, &cmd, 1, NULL, 0);
 }
 
+static int
+ax203_eeprom_clear_block_protection(Camera *camera)
+{
+	char cmd[2];
+
+	cmd[0] = SPI_EEPROM_WRSR;
+	cmd[1] = 0;
+
+	return ax203_send_eeprom_cmd (camera, 1, cmd, sizeof(cmd), NULL, 0);
+}
+
 static int
 ax203_eeprom_erase_4k_sector(Camera *camera, int address)
 {
@@ -290,12 +348,28 @@ static int
 ax203_eeprom_wait_ready(Camera *camera)
 {
 	char cmd = SPI_EEPROM_RDSR; /* Read status */
-	char buf[64]; /* Do as windows does, read the status word 64 times,
-			 then check */
+	char buf[64];
+	int count = 0;
+
+	switch (camera->pl->frame_version) {
+	case AX203_FIRMWARE_3_3_x:
+	case AX203_FIRMWARE_3_4_x:
+	case AX206_FIRMWARE_3_5_x:
+		/* Do as windows does, read the status word 64 times,
+		   then check */
+		count = 64;
+		break;
+	case AX3003_FIRMWARE_3_5_x:
+		/* On the ax3003 contineously reading the status word
+		   does not work. */
+		count = 1;
+		break;
+	}
+
 	while (1) {
-		CHECK (ax203_send_eeprom_cmd (camera, 0, &cmd, 1, buf, 64))
+		CHECK (ax203_send_eeprom_cmd (camera, 0, &cmd, 1, buf, count))
 		/* We only need to check the last read */
-		if (!(buf[63] & 0x01)) /* Check write in progress bit */
+		if (!(buf[count - 1] & 0x01)) /* Check write in progress bit */
 			break; /* No write in progress, done waiting */
 	}
 	return GP_OK;
@@ -473,9 +547,10 @@ ax203_write_mem(Camera *camera, int offset,
 static int ax203_read_parameter_block(Camera *camera)
 {
 	uint8_t buf[32], expect[32];
-	int i, param_offset, resolution_offset, compression_offset = -1;
-	int abfs_start_offset, expect_size;
+	int i, param_offset = 0, resolution_offset = 0;
+	int compression_offset = -1, abfs_start_offset = 0, expect_size = 0;
 	const int valid_resolutions[][2] = {
+		{  96,  64 },
 		{ 120, 160 },
 		{ 128, 128 },
 		{ 128, 160 },
@@ -494,7 +569,7 @@ static int ax203_read_parameter_block(Camera *camera)
 	const uint8_t expect_35x[] = {
 		0x00, 0x00, 0, 0, 0, 0, 0, 0xd8 };
 
-	switch (camera->pl->firmware_version) {
+	switch (camera->pl->frame_version) {
 	case AX203_FIRMWARE_3_3_x:
 		param_offset = 0x50;
 		resolution_offset  = 2;
@@ -513,20 +588,41 @@ static int ax203_read_parameter_block(Camera *camera)
 		memcpy (expect, expect_34x, sizeof(expect_34x));
 		expect_size = sizeof(expect_34x);
 		break;
-	case AX203_FIRMWARE_3_5_x:
+	case AX206_FIRMWARE_3_5_x:
 		param_offset = 0x20;
 		abfs_start_offset  = 2;
 		resolution_offset  = 3;
 		memcpy (expect, expect_35x, sizeof(expect_35x));
 		expect_size = sizeof(expect_35x);
-		/* 3.5.x firmware has a fixed compression type */
-		camera->pl->compression_version = AX203_COMPRESSION_JPEG;
+		/* ax206 + 3.5.x firmware has a fixed compression type */
+		camera->pl->compression_version = AX206_COMPRESSION_JPEG;
 		break;
+	case AX3003_FIRMWARE_3_5_x:
+		i = ax3003_get_frame_id (camera);
+		if (i < 0) return i;
+		switch (i) {
+		case 0:
+			camera->pl->width  = 320;
+			camera->pl->height = 240;
+			break;
+		default:
+			gp_log (GP_LOG_ERROR, "ax203",
+				"unknown ax3003 frame id: %d", i);
+			return GP_ERROR_MODEL_NOT_FOUND;
+		}
+
+		i = ax3003_get_abfs_start (camera);
+		if (i < 0) return i;
+		camera->pl->fs_start = i;
+
+		camera->pl->compression_version = AX3003_COMPRESSION_JPEG;
+
+		goto verify_parameters;
 	}
 
 	CHECK (ax203_read_mem (camera, param_offset, buf, sizeof(buf)))
 
-	switch (camera->pl->firmware_version) {
+	switch (camera->pl->frame_version) {
 	case AX203_FIRMWARE_3_3_x:
 		/* 1 byte width / height */
 		camera->pl->width  = buf[resolution_offset    ];
@@ -535,13 +631,16 @@ static int ax203_read_parameter_block(Camera *camera)
 		expect[resolution_offset + 1] = camera->pl->height;
 		break;
 	case AX203_FIRMWARE_3_4_x:
-	case AX203_FIRMWARE_3_5_x:
+	case AX206_FIRMWARE_3_5_x:
 		/* 2 byte little endian width / height */
 		camera->pl->width  = le16atoh (buf + resolution_offset);
 		camera->pl->height = le16atoh (buf + resolution_offset + 2);
 		htole16a(expect + resolution_offset    , camera->pl->width);
 		htole16a(expect + resolution_offset + 2, camera->pl->height);
 		break;
+	case AX3003_FIRMWARE_3_5_x:
+		/* Never reached, here to silence a compiler warning */
+		break;
 	}
 
 	if (compression_offset != -1) {
@@ -573,6 +672,7 @@ static int ax203_read_parameter_block(Camera *camera)
 		return GP_ERROR_MODEL_NOT_FOUND;
 	}
 
+verify_parameters:
 	for (i = 0; valid_resolutions[i][0]; i++)
 		if (valid_resolutions[i][0] == camera->pl->width &&
 		    valid_resolutions[i][1] == camera->pl->height)
@@ -596,7 +696,8 @@ static int ax203_read_parameter_block(Camera *camera)
 		  camera->pl->compression_version, camera->pl->fs_start);
 
 	/* Set JPEG compression parameters based on the found resolution */
-	if (camera->pl->width % 16 || camera->pl->height % 16) {
+	if (camera->pl->compression_version == AX206_COMPRESSION_JPEG &&
+	    (camera->pl->width % 16 || camera->pl->height % 16)) {
 		gp_log (GP_LOG_DEBUG, "ax203", "height or width not a "
 			"multiple of 16, forcing 1x subsampling");
 		camera->pl->jpeg_uv_subsample = 1;
@@ -636,7 +737,8 @@ ax203_filesize(Camera *camera)
 		return camera->pl->width * camera->pl->height;
 	case AX203_COMPRESSION_YUV_DELTA:
 		return camera->pl->width * camera->pl->height * 3 / 4;
-	case AX203_COMPRESSION_JPEG:
+	case AX206_COMPRESSION_JPEG:
+	case AX3003_COMPRESSION_JPEG:
 		/* Variable size */
 		return 0;
 	}
@@ -647,13 +749,16 @@ ax203_filesize(Camera *camera)
 static int
 ax203_max_filecount(Camera *camera)
 {
-	switch (camera->pl->firmware_version) {
+	switch (camera->pl->frame_version) {
 	case AX203_FIRMWARE_3_3_x:
 	case AX203_FIRMWARE_3_4_x:
 		return (AX203_ABFS_SIZE - AX203_ABFS_FILE_OFFSET (0)) / 2;
-	case AX203_FIRMWARE_3_5_x:
+	case AX206_FIRMWARE_3_5_x:
 		return (AX203_ABFS_SIZE - AX206_ABFS_FILE_OFFSET (0)) /
-			sizeof(struct ax203_v3_5_x_raw_fileinfo);
+			sizeof(struct ax206_v3_5_x_raw_fileinfo);
+	case AX3003_FIRMWARE_3_5_x:
+		return (AX203_ABFS_SIZE - AX3003_ABFS_FILE_OFFSET (0)) /
+			sizeof(struct ax3003_v3_5_x_raw_fileinfo);
 	}
 	/* Never reached */
 	return GP_ERROR_NOT_SUPPORTED;	
@@ -744,10 +849,10 @@ int ax203_write_v3_3_x_v3_4_x_fileinfo(Camera *camera, int idx,
 }
 
 static
-int ax203_read_v3_5_x_fileinfo(Camera *camera, int idx,
+int ax206_read_v3_5_x_fileinfo(Camera *camera, int idx,
 	struct ax203_fileinfo *fileinfo)
 {
-	struct ax203_v3_5_x_raw_fileinfo raw;
+	struct ax206_v3_5_x_raw_fileinfo raw;
 
 	CHECK (ax203_read_mem (camera,
 			       camera->pl->fs_start +
@@ -762,10 +867,10 @@ int ax203_read_v3_5_x_fileinfo(Camera *camera, int idx,
 }
 
 static
-int ax203_write_v3_5_x_fileinfo(Camera *camera, int idx,
+int ax206_write_v3_5_x_fileinfo(Camera *camera, int idx,
 	struct ax203_fileinfo *fileinfo)
 {
-	struct ax203_v3_5_x_raw_fileinfo raw;
+	struct ax206_v3_5_x_raw_fileinfo raw;
 
 	raw.present = fileinfo->present;
 	raw.address = htole32(fileinfo->address);
@@ -779,16 +884,67 @@ int ax203_write_v3_5_x_fileinfo(Camera *camera, int idx,
 	return GP_OK;
 }
 
+static
+int ax3003_read_v3_5_x_fileinfo(Camera *camera, int idx,
+	struct ax203_fileinfo *fileinfo)
+{
+	struct ax3003_v3_5_x_raw_fileinfo raw;
+
+	CHECK (ax203_read_mem (camera,
+			       camera->pl->fs_start +
+					AX3003_ABFS_FILE_OFFSET (idx),
+			       &raw, sizeof(raw)))
+
+	fileinfo->present = raw.address && raw.size;
+	fileinfo->address = be16toh (raw.address) * 0x100;
+	fileinfo->size    = be16toh (raw.size) * 0x100;
+
+	return GP_OK;
+}
+
+static
+int ax3003_write_v3_5_x_fileinfo(Camera *camera, int idx,
+	struct ax203_fileinfo *fileinfo)
+{
+	struct ax3003_v3_5_x_raw_fileinfo raw;
+
+	if (fileinfo->address & 0xff) {
+		gp_log (GP_LOG_ERROR, "ax203", "LSB of address is not 0");
+		return GP_ERROR_BAD_PARAMETERS;
+	}
+
+	if (fileinfo->size & 0xff) {
+		gp_log (GP_LOG_ERROR, "ax203", "LSB of size is not 0");
+		return GP_ERROR_BAD_PARAMETERS;
+	}
+
+	if (fileinfo->present) {
+		raw.address = htobe16(fileinfo->address / 0x100);
+		raw.size    = htobe16(fileinfo->size / 0x100);
+	} else {
+		raw.address = 0;
+		raw.size    = 0;
+	}
+
+	CHECK (ax203_write_mem (camera,
+				camera->pl->fs_start +
+					AX3003_ABFS_FILE_OFFSET (idx),
+				&raw, sizeof(raw)))
+
+	return GP_OK;
+}
+
 /***************** Begin "public" functions *****************/
 
 int
 ax203_read_filecount(Camera *camera)
 {
-	switch (camera->pl->firmware_version) {
+	switch (camera->pl->frame_version) {
 	case AX203_FIRMWARE_3_3_x:
 	case AX203_FIRMWARE_3_4_x:
 		return ax203_read_v3_3_x_v3_4_x_filecount (camera);
-	case AX203_FIRMWARE_3_5_x:
+	case AX206_FIRMWARE_3_5_x:
+	case AX3003_FIRMWARE_3_5_x:
 		return ax203_read_v3_5_x_filecount (camera);;
 	}
 	/* Never reached */
@@ -807,11 +963,12 @@ ax203_update_filecount(Camera *camera)
 			count = i + 1;
 	}
 
-	switch (camera->pl->firmware_version) {
+	switch (camera->pl->frame_version) {
 	case AX203_FIRMWARE_3_3_x:
 	case AX203_FIRMWARE_3_4_x:
 		return ax203_write_v3_3_x_v3_4_x_filecount (camera, count);
-	case AX203_FIRMWARE_3_5_x:
+	case AX206_FIRMWARE_3_5_x:
+	case AX3003_FIRMWARE_3_5_x:
 		return ax203_write_v3_5_x_filecount (camera, count);
 	}
 	/* Never reached */
@@ -824,13 +981,15 @@ int ax203_read_fileinfo(Camera *camera, int idx,
 {
 	CHECK (ax203_check_file_index (camera, idx))
 
-	switch (camera->pl->firmware_version) {
+	switch (camera->pl->frame_version) {
 	case AX203_FIRMWARE_3_3_x:
 	case AX203_FIRMWARE_3_4_x:
 		return ax203_read_v3_3_x_v3_4_x_fileinfo (camera, idx,
 							  fileinfo);
-	case AX203_FIRMWARE_3_5_x:
-		return ax203_read_v3_5_x_fileinfo (camera, idx, fileinfo);
+	case AX206_FIRMWARE_3_5_x:
+		return ax206_read_v3_5_x_fileinfo (camera, idx, fileinfo);
+	case AX3003_FIRMWARE_3_5_x:
+		return ax3003_read_v3_5_x_fileinfo (camera, idx, fileinfo);
 	}
 	/* Never reached */
 	return GP_ERROR_NOT_SUPPORTED;	
@@ -840,13 +999,15 @@ static
 int ax203_write_fileinfo(Camera *camera, int idx,
 	struct ax203_fileinfo *fileinfo)
 {
-	switch (camera->pl->firmware_version) {
+	switch (camera->pl->frame_version) {
 	case AX203_FIRMWARE_3_3_x:
 	case AX203_FIRMWARE_3_4_x:
 		return ax203_write_v3_3_x_v3_4_x_fileinfo (camera, idx,
 							   fileinfo);
-	case AX203_FIRMWARE_3_5_x:
-		return ax203_write_v3_5_x_fileinfo (camera, idx, fileinfo);
+	case AX206_FIRMWARE_3_5_x:
+		return ax206_write_v3_5_x_fileinfo (camera, idx, fileinfo);
+	case AX3003_FIRMWARE_3_5_x:
+		return ax3003_write_v3_5_x_fileinfo (camera, idx, fileinfo);
 	}
 	/* Never reached */
 	return GP_ERROR_NOT_SUPPORTED;	
@@ -869,6 +1030,12 @@ ax203_decode_image(Camera *camera, char *src, int src_size, int **dest)
 	int ret;
 	unsigned int x, y, width, height;
 	unsigned char *components[3];
+#ifdef HAVE_LIBJPEG
+	struct jpeg_decompress_struct dinfo;
+	struct jpeg_error_mgr jderr;
+	JSAMPLE row[camera->pl->width * 3];
+	JSAMPROW row_pointer[1] = { row };
+#endif
 
 	switch (camera->pl->compression_version) {
 	case AX203_COMPRESSION_YUV:
@@ -879,7 +1046,10 @@ ax203_decode_image(Camera *camera, char *src, int src_size, int **dest)
 		ax203_decode_yuv_delta (src, dest, camera->pl->width,
 					camera->pl->height);
 		return GP_OK;
-	case AX203_COMPRESSION_JPEG:
+	case AX206_COMPRESSION_JPEG:
+		/* Note this uses a modified tinyjpeg which was modified to
+		   handle the AX206' custom JPEG format (the AX3003 uses
+		   normal JPEG compression). */
 		if (!camera->pl->jdec) {
 			camera->pl->jdec = tinyjpeg_init ();
 			if (!camera->pl->jdec)
@@ -919,6 +1089,42 @@ ax203_decode_image(Camera *camera, char *src, int src_size, int **dest)
 			}
 		}
 		return GP_OK;
+	case AX3003_COMPRESSION_JPEG:
+#ifdef HAVE_LIBJPEG
+		dinfo.err = jpeg_std_error (&jderr);
+		jpeg_create_decompress (&dinfo);
+		jpeg_mem_src (&dinfo, (unsigned char *)src, src_size);
+		jpeg_read_header (&dinfo, TRUE);
+		jpeg_start_decompress (&dinfo);
+		if (dinfo.output_width  != camera->pl->width ||
+		    dinfo.output_height != camera->pl->height ||
+		    dinfo.output_components != 3 ||
+		    dinfo.out_color_space != JCS_RGB) {
+			gp_log (GP_LOG_ERROR, "ax203",
+				"Wrong JPEG header parameters: %dx%d, "
+				"%d components, colorspace: %d",
+				dinfo.output_width, dinfo.output_height,
+				dinfo.output_components,
+				dinfo.out_color_space);
+			return GP_ERROR_CORRUPTED_DATA;
+		}
+
+		for (y = 0; y < dinfo.output_height; y++) {
+			jpeg_read_scanlines (&dinfo, row_pointer, 1);
+			for (x = 0; x < dinfo.output_width; x++) {
+				dest[y][x] = gdTrueColor (row[x * 3 + 0],
+							  row[x * 3 + 1],
+							  row[x * 3 + 2]);
+			}
+		}
+		jpeg_finish_decompress (&dinfo);
+		jpeg_destroy_decompress (&dinfo);
+		return GP_OK;
+#else
+		gp_log(GP_LOG_ERROR,"ax203", "jpeg decompression not supported - no libjpeg during build");
+		return GP_ERROR_NOT_SUPPORTED;
+#endif
+		break;
 	}
 #endif
 	/* Never reached */
@@ -931,9 +1137,18 @@ ax203_encode_image(Camera *camera, int **src, char *dest, int dest_size)
 {
 #ifdef HAVE_GD
 	int size = ax203_filesize (camera);
+#ifdef HAVE_LIBJPEG
+	int x, y;
+	struct jpeg_compress_struct cinfo;
+	struct jpeg_error_mgr jcerr;
+	JOCTET *jpeg_dest = NULL;
+	unsigned long jpeg_size = 0;
+	JSAMPLE row[camera->pl->width * 3];
+	JSAMPROW row_pointer[1] = { row };
+#endif
 
 	if (dest_size < size)
-		return GP_ERROR_BAD_PARAMETERS;
+		return GP_ERROR_FIXED_LIMIT_EXCEEDED;
 
 	switch (camera->pl->compression_version) {
 	case AX203_COMPRESSION_YUV:
@@ -944,11 +1159,48 @@ ax203_encode_image(Camera *camera, int **src, char *dest, int dest_size)
 		ax203_encode_yuv_delta (src, dest, camera->pl->width,
 					camera->pl->height);
 		return size;
-	case AX203_COMPRESSION_JPEG:
-		return ax203_compress_jpeg (camera, src,
+	case AX206_COMPRESSION_JPEG:
+		return ax206_compress_jpeg (camera, src,
 					    (uint8_t *)dest, dest_size,
 					    camera->pl->width,
 					    camera->pl->height);
+	case AX3003_COMPRESSION_JPEG:
+#ifdef HAVE_LIBJPEG
+		cinfo.err = jpeg_std_error (&jcerr);
+		jpeg_create_compress (&cinfo);
+		jpeg_mem_dest (&cinfo, &jpeg_dest, &jpeg_size);
+		cinfo.image_width = camera->pl->width;
+		cinfo.image_height = camera->pl->height;
+		cinfo.input_components = 3;
+		cinfo.in_color_space = JCS_RGB;
+		jpeg_set_defaults (&cinfo);
+		jpeg_start_compress (&cinfo, TRUE);
+		for (y = 0; y < cinfo.image_height; y++) {
+			for (x = 0; x < cinfo.image_width; x++) {
+				int p = src[y][x];
+				row[x * 3 + 0] = gdTrueColorGetRed(p);
+				row[x * 3 + 1] = gdTrueColorGetGreen(p);
+				row[x * 3 + 2] = gdTrueColorGetBlue(p);
+			}
+			jpeg_write_scanlines (&cinfo, row_pointer, 1);
+		}
+		jpeg_finish_compress (&cinfo);
+		jpeg_destroy_compress (&cinfo);
+
+		if (jpeg_size > dest_size) {
+			free (jpeg_dest);
+			gp_log (GP_LOG_ERROR, "ax203",
+				"JPEG is bigger then buffer");
+			return GP_ERROR_FIXED_LIMIT_EXCEEDED;
+		}
+		memcpy (dest, jpeg_dest, jpeg_size);
+		free (jpeg_dest);
+		/* Round size up to a multiple of 256 because of ax3003
+		   abfs size granularity. */
+                return (jpeg_size + 0xff) & ~0xff;
+#else
+		return GP_ERROR_NOT_SUPPORTED;
+#endif
 	}
 	/* Never reached */
 #endif
@@ -1086,14 +1338,17 @@ ax203_build_used_mem_table(Camera *camera, struct ax203_fileinfo *table)
 
 	/* The beginning of the memory is used by the CD image and stuff */
 	fileinfo.address = 0;
-	switch (camera->pl->firmware_version) {
+	switch (camera->pl->frame_version) {
 	case AX203_FIRMWARE_3_3_x:
 	case AX203_FIRMWARE_3_4_x:
 		fileinfo.size = camera->pl->fs_start + AX203_PICTURE_OFFSET;
 		break;
-	case AX203_FIRMWARE_3_5_x:
+	case AX206_FIRMWARE_3_5_x:
 		fileinfo.size = camera->pl->fs_start + AX206_PICTURE_OFFSET;
 		break;
+	case AX3003_FIRMWARE_3_5_x:
+		fileinfo.size = camera->pl->fs_start + AX3003_PICTURE_OFFSET;
+		break;
 	}
 	fileinfo.present = 1;
 	table[found++] = fileinfo;
@@ -1109,7 +1364,17 @@ ax203_build_used_mem_table(Camera *camera, struct ax203_fileinfo *table)
 
 	/* Add a last memory used region which starts at the end of memory
 	   the size does not matter as this is the last entry. */
-	fileinfo.address = camera->pl->mem_size;
+	switch (camera->pl->frame_version) {
+	case AX203_FIRMWARE_3_3_x:
+	case AX203_FIRMWARE_3_4_x:
+	case AX206_FIRMWARE_3_5_x:
+		fileinfo.address = camera->pl->mem_size;
+		break;
+	case AX3003_FIRMWARE_3_5_x:
+		fileinfo.address = camera->pl->mem_size - AX3003_BL_SIZE;
+		break;
+	}
+
 	fileinfo.size    = 0;
 	fileinfo.present = 1;
 	table[found++] = fileinfo;
@@ -1223,14 +1488,17 @@ ax203_delete_all(Camera *camera)
 	char buf[AX203_ABFS_SIZE];
 	int size, file0_offset = 0;
 
-	switch (camera->pl->firmware_version) {
+	switch (camera->pl->frame_version) {
 	case AX203_FIRMWARE_3_3_x:
 	case AX203_FIRMWARE_3_4_x:
 		file0_offset = AX203_ABFS_FILE_OFFSET (0);
 		break;
-	case AX203_FIRMWARE_3_5_x:
+	case AX206_FIRMWARE_3_5_x:
 		file0_offset = AX206_ABFS_FILE_OFFSET (0);
 		break;
+	case AX3003_FIRMWARE_3_5_x:
+		file0_offset = AX3003_ABFS_FILE_OFFSET (0);
+		break;
 	}
 
 	size = AX203_ABFS_SIZE - file0_offset;
@@ -1287,6 +1555,40 @@ ax203_commit_block_64k(Camera *camera, int bss)
 	return GP_OK;
 }
 
+static int
+ax203_commit_block_ax3003(Camera *camera, int bss)
+{
+	int block_sector_size = SPI_EEPROM_BLOCK_SIZE / SPI_EEPROM_SECTOR_SIZE;
+	int i, address = bss * SPI_EEPROM_SECTOR_SIZE;
+
+	/* Make sure we have read the entire block before erasing it !! */
+	for (i = 0; i < block_sector_size; i++)
+		CHECK (ax203_check_sector_present (camera, bss + i))
+
+	if (!camera->pl->block_protection_removed) {
+		CHECK (ax203_eeprom_write_enable (camera))
+		CHECK (ax203_eeprom_clear_block_protection (camera))
+		CHECK (ax203_eeprom_wait_ready (camera))
+		camera->pl->block_protection_removed = 1;
+	}
+
+	/* Erase the block */
+	CHECK (ax203_erase64k_sector (camera, bss))
+
+	/* And program the block in one large 64k page write, the ax3003
+	   probably emulates this in firmware, to avoid usb bus overhead. */
+	CHECK (ax203_eeprom_write_enable (camera))
+	CHECK (ax203_eeprom_program_page (camera, address,
+					  camera->pl->mem + address,
+					  SPI_EEPROM_BLOCK_SIZE))
+	CHECK (ax203_eeprom_wait_ready (camera))
+
+	for (i = 0; i < block_sector_size; i++)
+		camera->pl->sector_dirty[bss + i] = 0;
+
+	return GP_OK;
+}
+
 int
 ax203_commit(Camera *camera)
 {
@@ -1309,10 +1611,12 @@ ax203_commit(Camera *camera)
 		if (!dirty_sectors)
 			continue;
 
+		if (camera->pl->frame_version == AX3003_FIRMWARE_3_5_x)
+			CHECK (ax203_commit_block_ax3003 (camera, i))
 		/* There are 16 4k sectors per 64k block, when we need to
 		   program 12 or more sectors, programming the entire block
 		   becomes faster */
-		if (dirty_sectors < 12 && camera->pl->has_4k_sectors)
+		else if (dirty_sectors < 12 && camera->pl->has_4k_sectors)
 			CHECK (ax203_commit_block_4k (camera, i))
 		else
 			CHECK (ax203_commit_block_64k (camera, i))
@@ -1381,12 +1685,6 @@ ax203_open_device(Camera *camera)
 		  ax203_eeprom_info[i].name, camera->pl->mem_size,
 		  camera->pl->has_4k_sectors);
 
-	if (camera->pl->has_4k_sectors == -1) {
-		gp_log (GP_LOG_ERROR, "ax203", "%s has an unknown sector size",
-			ax203_eeprom_info[i].name);
-		return GP_ERROR_NOT_SUPPORTED;
-	}
-
 	return ax203_init (camera);
 }
 
