@@ -42,7 +42,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdint.h>
+#include <_stdint.h>
 #include <errno.h>
 
 #include "tinyjpeg.h"
