@@ -21,7 +21,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <stdint.h>
+#include <_stdint.h>
 #include <stdlib.h>
 #include <time.h>
 #include <fcntl.h>
@@ -38,19 +38,6 @@ struct image_table_entry {
 	char name[ST2205_FILENAME_LENGTH + 1]; /* Image name (11 bytes) */
 } __attribute__((packed));
 
-struct image_header {
-	uint8_t marker;   /* Always 0xF5 */
-	uint16_t width;	  /* big endian */
-	uint16_t height;  /* big endian */
-	uint16_t blocks;  /* number of 8x8 blocks in the image, big endian */
-	uint8_t shuffle_table; /* shuffle table idx (for transition effects) */
-	uint8_t unknown2; /* Unknown usually 0x04 */
-	uint8_t unknown3; /* shuffle related, must have a special value depend
-			     on the pattern see the table in st2205_init */
-	uint16_t length;  /* length of the data *following* the header (be) */
-	uint8_t unknown4[4]; /* Always 4x 0x00 (padding ?) */
-} __attribute__((packed));
-
 /* The st2205 port driver's write and read functions require page aligned
    buffers, as they use O_DIRECT. */
 static char *st2205_malloc_page_aligned(int size)
@@ -219,7 +206,7 @@ st2205_detect_mem_size(Camera *camera)
 	if (ret)
 		return ret;
 
-	for (i = 0; i < 2; i++) {
+	for (i = 0; i < 3; i++) {
 		ret = st2205_read_block(camera,
 					(524288 / ST2205_BLOCK_SIZE) << i,
 					buf1);
@@ -230,7 +217,6 @@ st2205_detect_mem_size(Camera *camera)
 	}
 
 	camera->pl->mem_size = 524288 << i;	
-	GP_DEBUG ("Detect %d bytes of flash memory", camera->pl->mem_size);
 
 	st2205_free_page_aligned(buf0, ST2205_BLOCK_SIZE);
 	st2205_free_page_aligned(buf1, ST2205_BLOCK_SIZE);
@@ -380,7 +366,6 @@ st2205_add_picture(Camera *camera, int idx, const char *filename,
 {
 	int count;
 	struct image_table_entry entry;
-	struct image_header header;
 
 	count = st2205_read_file_count(camera);
 	if (count < 0) return count;
@@ -408,26 +393,14 @@ st2205_add_picture(Camera *camera, int idx, const char *filename,
 		   seem to care about this, but the windows software does this,
 		   so lets do it too. */
 		memset (&entry, 0, sizeof(entry));
-		entry.address = htole32 (start + size + sizeof(header));
+		entry.address = htole32 (start + size);
 		CHECK (st2205_write_mem (camera, ST2205_FILE_OFFSET (count),
 					 &entry, sizeof(entry)))
 	}
 
 	CHECK (st2205_update_fat_checksum (camera))
 	CHECK (st2205_copy_fat (camera))
-
-	memset(&header, 0, sizeof(header));
-	header.marker = ST2205_HEADER_MARKER;
-	header.width  = htobe16(camera->pl->width);
-	header.height = htobe16(camera->pl->height);
-	header.blocks = htobe16((camera->pl->width * camera->pl->height) / 64);
-	header.shuffle_table = shuffle;
-	header.unknown2 = 0x04;
-	header.unknown3 = camera->pl->unknown3[shuffle];
-	header.length = htobe16(size);
-	CHECK (st2205_write_mem (camera, start,
-				 &header, sizeof(header)))
-	CHECK (st2205_write_mem (camera, start + sizeof(header), buf, size))
+	CHECK (st2205_write_mem (camera, start,	buf, size))
 
 	/* Let the caller know at which index we stored the table entry */
 	return idx;
@@ -482,12 +455,13 @@ st2205_get_filenames(Camera *camera, st2205_filename *names)
 }
 
 int
-st2205_read_file(Camera *camera, int idx, int **rgb24)
+st2205_read_raw_file(Camera *camera, int idx, unsigned char **raw)
 {
-	unsigned char *src;
 	struct image_table_entry entry;
-	struct image_header header;
-	int ret, count;
+	struct st2205_image_header header;
+	int ret, count, size;
+
+	*raw = NULL;
 
 	count = st2205_read_file_count(camera);
 	if (count < 0) return count;
@@ -511,46 +485,68 @@ st2205_read_file(Camera *camera, int idx, int **rgb24)
 
 	GP_DEBUG ("file: %d start at: %08x\n", idx, entry.address);
 
-	CHECK (st2205_read_mem (camera, entry.address,
-				&header, sizeof(header)))
+	if (camera->pl->compressed) {
+		CHECK (st2205_read_mem (camera, entry.address,
+					&header, sizeof(header)))
 
-	if (header.marker != ST2205_HEADER_MARKER) {
-		gp_log (GP_LOG_ERROR, "st2205", "invalid header magic");
-		return GP_ERROR_CORRUPTED_DATA;
-	}
+		if (header.marker != ST2205_HEADER_MARKER) {
+			gp_log (GP_LOG_ERROR, "st2205", "invalid header magic");
+			return GP_ERROR_CORRUPTED_DATA;
+		}
 
-	BE16TOH(header.width);
-	BE16TOH(header.height);
-	BE16TOH(header.length);
-	BE16TOH(header.blocks);
+		BE16TOH(header.width);
+		BE16TOH(header.height);
+		BE16TOH(header.length);
+		BE16TOH(header.blocks);
 
-	if ((header.width != camera->pl->width) ||
-	    (header.height != camera->pl->height)) {
-		gp_log (GP_LOG_ERROR, "st2205",
-			"picture size does not match frame size.");
-		return GP_ERROR_CORRUPTED_DATA;
-	}
+		if ((header.width != camera->pl->width) ||
+		    (header.height != camera->pl->height)) {
+			gp_log (GP_LOG_ERROR, "st2205",
+				"picture size does not match frame size.");
+			return GP_ERROR_CORRUPTED_DATA;
+		}
 
-	if (((header.width / 8) * (header.height / 8)) != header.blocks) {
-		gp_log (GP_LOG_ERROR, "st2205", "invalid block count");
-		return GP_ERROR_CORRUPTED_DATA;
-	}
+		if (((header.width / 8) * (header.height / 8)) != header.blocks) {
+			gp_log (GP_LOG_ERROR, "st2205", "invalid block count");
+			return GP_ERROR_CORRUPTED_DATA;
+		}
 
-	GP_DEBUG ("file: %d header read, size: %dx%d, length: %d bytes\n",
-		  idx, header.width, header.height, header.length);
+		GP_DEBUG ("file: %d header read, size: %dx%d, length: %d bytes\n",
+			  idx, header.width, header.height, header.length);
 
-	src = malloc(header.length);
-	if (!src) {
+		size = header.length + sizeof (header);
+	} else
+		size = camera->pl->width * camera->pl->height * 2;
+
+	*raw = malloc (size);
+	if (!*raw) {
 		gp_log (GP_LOG_ERROR, "st2205", "allocating memory");
 		return GP_ERROR_NO_MEMORY;
 	}
 
-	ret = st2205_read_mem (camera, entry.address + sizeof(header), src,
-			       header.length);
-	if (ret < 0) { free(src); return ret; }
+	ret = st2205_read_mem (camera, entry.address, *raw, size);
+	if (ret < 0) {
+		free (*raw);
+		*raw = NULL;
+		return ret;
+	}
+
+	return size;
+}
+
+int
+st2205_read_file(Camera *camera, int idx, int **rgb24)
+{
+	int ret;
+	unsigned char *src;
+
+	CHECK (st2205_read_raw_file (camera, idx, &src))
+
+	if (camera->pl->compressed)
+		ret = st2205_decode_image (camera->pl, src, rgb24);
+	else
+		ret = st2205_rgb565_to_rgb24 (camera->pl, src, rgb24);
 
-	ret = st2205_decode_image (camera->pl, src, header.length, rgb24,
-				   header.shuffle_table);
 	free(src);
 
 	return ret;
@@ -563,12 +559,14 @@ st2205_real_write_file(Camera *camera,
 {
 	int size, count;
 	struct image_table_entry entry;
-	struct image_header header;
+	struct st2205_image_header header;
 	int i, start, end, hole_start = 0, hole_idx = 0;
 
-	size = st2205_code_image (camera->pl, rgb24, buf, shuffle,
-				  allow_uv_corr);
-	if (size < 0) return size;
+	if (camera->pl->compressed)
+		size = st2205_code_image (camera->pl, rgb24, buf, shuffle,
+					  allow_uv_corr);
+	else
+		size = st2205_rgb24_to_rgb565 (camera->pl, rgb24, buf);
 
 	count = st2205_read_file_count (camera);
 	if (count < 0) return count;
@@ -594,12 +592,16 @@ st2205_real_write_file(Camera *camera,
 
 			start = entry.address;
 			if (entry.present) {
-				CHECK (st2205_read_mem (camera, start,
-							&header,
-							sizeof(header)))
+				if (camera->pl->compressed) {
+					CHECK (st2205_read_mem (camera, start,
+							      &header,
+							      sizeof(header)))
 
-				BE16TOH(header.length);
-				end = start + sizeof(header) + header.length;
+					BE16TOH(header.length);
+					end = start + sizeof(header) + 
+					      header.length;
+				} else
+					end = start + size;
 			}
 		}
 
@@ -610,8 +612,8 @@ st2205_real_write_file(Camera *camera,
 				int hole_size = start - hole_start;
 				GP_DEBUG ("found a hole at: %08x, of %d bytes "
 					  "(need %d)\n", hole_start, hole_size,
-					  (int)(size + sizeof(header)));
-				if (hole_size < (size + sizeof(header))) {
+					  size);
+				if (hole_size < size) {
 					/* Too small, start searching
 					   for the next hole */
 					hole_start = 0;
@@ -637,7 +639,7 @@ st2205_real_write_file(Camera *camera,
 	}
 	
 	/* No freespace found, try again with uv correction tables disabled */
-	if (allow_uv_corr)
+	if (camera->pl->compressed && allow_uv_corr)
 		return st2205_real_write_file (camera, filename, rgb24, buf,
 					       shuffle, 0);
 
@@ -651,7 +653,7 @@ st2205_write_file(Camera *camera,
 	const char *filename, int **rgb24)
 {
 	/* The buffer must be large enough for the worst case scenario */
-	unsigned char buf[(camera->pl->width * camera->pl->height / 64) * 64];
+	unsigned char buf[camera->pl->width * camera->pl->height * 2];
 	int shuffle;
 
 	shuffle = (long long)rand_r(&camera->pl->rand_seed) *
@@ -704,6 +706,7 @@ st2205_delete_all(Camera *camera)
 	/* Mark the memory block we've directly manipulated dirty. */
 	camera->pl->block_dirty[0] = 1;
 
+	CHECK (st2205_write_file_count (camera, 0))
 	CHECK (st2205_update_fat_checksum (camera))
 	CHECK (st2205_copy_fat (camera))
 
@@ -715,6 +718,10 @@ st2205_set_time_and_date(Camera *camera, struct tm *t)
 {
 	uint8_t *buf = (uint8_t *)camera->pl->buf;
 
+	/* We cannot do this when operating on a dump */
+	if (camera->pl->mem_dump)
+		return GP_OK;
+
 	memset(buf, 0, 512);
 	buf[0] = 6; /* cmd 6 set time */
 	htobe16a (buf + 1, t->tm_year + 1900);
@@ -781,6 +788,7 @@ st2205_init(Camera *camera)
 	uint16_t *lookup_src, *dest;
 	uint8_t *shuffle_src;
 	int x, y, i, j, shuffle_size, checksum, expected_checksum;
+	int is240x320 = 0;
 	const struct {
 		int width, height, no_tables, usable_tables;
 		unsigned char unknown3[8];
@@ -793,8 +801,7 @@ st2205_init(Camera *camera)
 		  { 0xff, 0xff, 0x01, 0x01, 0x01, 0x01, 0x01 },
 		  0x00025800 },
 		{ 120, 160, 7, 7,
-		  /* FIXME unknown3 needs to be verified */
-		  { 0xff, 0xff, 0x01, 0x01, 0x01, 0x01, 0x01 },
+		  { 0xff, 0xff, 0x04, 0x04, 0x04, 0x04, 0x04 },
 		  0x00030570 },
 		{ 96, 64, 7, 7,
 		  { 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00 },
@@ -813,6 +820,7 @@ st2205_init(Camera *camera)
 		  ST2205_V2_PICTURE_START, 1 },
 		{ }
 	};
+	const int uncompressed_firmware_checksums[] = { 0x00ab02fc, 0 };
 
 	GP_DEBUG ("st2205_init called");
 
@@ -824,6 +832,9 @@ st2205_init(Camera *camera)
 		return GP_ERROR_IO;
 	}
 
+	if (camera->pl->width == 240 && camera->pl->height == 320)
+		is240x320 = 1;
+
 	shuffle_size = (camera->pl->width / 8) * (camera->pl->height / 8);
 	if (shuffle_size > ST2205_SHUFFLE_SIZE) {
 		gp_log (GP_LOG_ERROR, "st2205",
@@ -892,6 +903,9 @@ st2205_init(Camera *camera)
 		if (camera->pl->width	== shuffle_info[i].width &&
 				camera->pl->height == shuffle_info[i].height)
 			break;
+		if (is240x320 && shuffle_info[i].width == 120 &&
+				shuffle_info[i].height == 160)
+			break;
 		shuffle_src += (shuffle_info[i].width *
 				shuffle_info[i].height * 2 / 64) *
 			       (shuffle_info[i].no_tables - 2);
@@ -920,6 +934,23 @@ st2205_init(Camera *camera)
 			}
 			checksum += camera->pl->shuffle[j][i].x;
 			checksum += camera->pl->shuffle[j][i].y;
+			if (is240x320) {
+				camera->pl->shuffle[j][i].x *= 2;
+				camera->pl->shuffle[j][i].y *= 2;
+				camera->pl->shuffle[j][i + 1].x =
+					camera->pl->shuffle[j][i].x + 8;
+				camera->pl->shuffle[j][i + 1].y =
+					camera->pl->shuffle[j][i].y;
+				camera->pl->shuffle[j][i + 2].x =
+					camera->pl->shuffle[j][i].x;
+				camera->pl->shuffle[j][i + 2].y =
+					camera->pl->shuffle[j][i].y + 8;
+				camera->pl->shuffle[j][i + 3].x =
+					camera->pl->shuffle[j][i].x + 8;
+				camera->pl->shuffle[j][i + 3].y =
+					camera->pl->shuffle[j][i].y + 8;
+				i += 3;
+			}
 		}
 
 	if (checksum != expected_checksum) {
@@ -932,6 +963,29 @@ st2205_init(Camera *camera)
 
 	camera->pl->rand_seed = time(NULL);
 
+	/* Some 96x64 models don't use compression, unfortunately I've found
+	   no way to detect if this is the case, so we keep a list of firmware
+	   checksums to identify these. */
+	for (i = camera->pl->mem_size - camera->pl->firmware_size;
+	     i < camera->pl->mem_size; i += ST2205_BLOCK_SIZE)
+		CHECK (st2205_check_block_present (camera,
+						   i / ST2205_BLOCK_SIZE))
+	checksum = 0;
+	for (i = camera->pl->mem_size - camera->pl->firmware_size;
+	     i < camera->pl->mem_size; i++)
+		checksum += (uint8_t)camera->pl->mem[i];
+
+	GP_DEBUG ("firmware checksum: 0x%08x", checksum);
+
+	for (i = 0; uncompressed_firmware_checksums[i]; i++)
+		if (uncompressed_firmware_checksums[i] == checksum)
+			break;
+
+	if (!uncompressed_firmware_checksums[i])
+		camera->pl->compressed = 1;
+	else
+		camera->pl->compressed = 0;
+
 	return GP_OK;
 }
 
@@ -1008,3 +1062,63 @@ st2205_get_mem_size(Camera *camera)
 {
 	return camera->pl->mem_size;
 }
+
+int
+st2205_get_free_mem_size(Camera *camera)
+{
+	struct image_table_entry entry;
+	struct st2205_image_header header;
+	int i, count, start, end, hole_start = 0, free = 0;
+
+	count = st2205_read_file_count (camera);
+	if (count < 0) return count;
+
+	/* Find all holes in the memory and add their sizes together */
+	end = camera->pl->picture_start;
+	for (i = 0; i <= count; i++) {
+		/* Fake a present entry at the end of picture mem */
+		if (i == count) {
+			entry.present = 1;
+			start = camera->pl->mem_size -
+				camera->pl->firmware_size;
+			/* If the last entry in the "FAT" was present, we need
+			   to set hole_start to the end of the last picture */
+			if (!hole_start)
+				hole_start = end;
+		} else {
+			CHECK (st2205_read_mem (camera, ST2205_FILE_OFFSET (i),
+						&entry, sizeof(entry)))
+
+			start = entry.address;
+			if (entry.present) {
+				if (camera->pl->compressed) {
+					CHECK (st2205_read_mem (camera, start,
+							      &header,
+							      sizeof(header)))
+
+					BE16TOH(header.length);
+					end = start + sizeof(header) +
+					      header.length;
+				} else {
+					end = start +
+					      camera->pl->width *
+					      camera->pl->height * 2;
+				}
+			}
+		}
+
+		/* If we have a hole start address look for present entries (so
+		   a hole end address), else look for non present entries */
+		if (hole_start) {
+			if (entry.present) {
+				free += start - hole_start;
+				hole_start = 0;
+			}
+		} else {
+			if (!entry.present)
+				hole_start = end;
+		}
+	}
+
+	return free;
+}
