@@ -86,10 +86,6 @@ gp_port_library_list (GPPortInfoList *list)
 
 	/* default port first */
 	info.type = GP_PORT_USB;
-	strcpy (info.name, "Universal Serial Bus");
-	strcpy (info.path, "usb:");
-	CHECK (gp_port_info_list_append (list, info));
-
 	/* generic matcher. This will catch passed XXX,YYY entries for instance. */
 	memset (info.name, 0, sizeof(info.name));
 	strcpy (info.path, "^usb:");
@@ -142,11 +138,14 @@ gp_port_library_list (GPPortInfoList *list)
 		bus = bus->next;
 	}
 
+#if 0
+	/* Do not filter out usb:XXX,YYY, we want it for consistency... */
 	/* If we already added usb:, and have 0 or 1 devices we have nothing to do.
 	 * This should be the standard use case.
 	 */
 	if (nrofdevices <= 1) 
 		return (GP_OK);
+#endif
 
 	/* Redo the same bus/device walk, but now add the ports with usb:x,y notation,
 	 * so we can address all USB devices.
@@ -193,6 +192,13 @@ gp_port_library_list (GPPortInfoList *list)
 		}
 		bus = bus->next;
 	}
+	/* This will only be added if no other device was ever added.
+	 * Users doing "usb:" usage will enter the regular expression matcher case. */
+	if (nrofdevices == 0) {
+		strcpy (info.name, "Universal Serial Bus");
+		strcpy (info.path, "usb:");
+		CHECK (gp_port_info_list_append (list, info));
+	}
 	return (GP_OK);
 }
 
